(* vgatest.c translated into O'Caml, using ocamlsvga.

	/* From VGAlib, changed for svgalib */
	/* partially copyrighted (C) 1993 by Hartmut Schirmer */

*)

(* $Id: vgatest.ml,v 1.2 1997/03/24 22:23:20 jc Exp $ *)

open Vga

let line = String.create (2048 * 3);;

let lineset i n = line.[i] <- Char.chr n;;

let rec getchar () =
  let d = getkey () in
  if d <> 0 then d else getchar ()
;; 

let testmode mode =

  Vga.setmode mode;
  
  let modeinfo = Vga.getmodeinfo mode in
    
  Printf.printf "Width: %d  Height: %d  Colors: %d\n"
    modeinfo.width 
    modeinfo.height 
    modeinfo.colors ;
  Printf.printf "DisplayStartRange: %xh  Maxpixels: %d  Blit: %s\n"
    modeinfo.startaddressrange
    modeinfo.maxpixels
    (if List.mem HAVE_BITBLIT modeinfo.haveblit then "YES" else "NO") ;
    
  Vga.screenoff ();

  let xmax = Vga.getxdim() - 1
  and ymax = Vga.getydim() - 1 in

    Vga.setcolor (Vga.white());
    Vga.drawline 0 0 xmax 0;
    Vga.drawline xmax 0 xmax ymax;
    Vga.drawline xmax ymax 0 ymax;
    Vga.drawline 0 ymax 0 0;
    
    for i = 0 to 15 do 
      Vga.setegacolor i;
      Vga.drawline (10 + i * 5) 10 (90 + i * 5) 90
    done ; 
    for i = 0 to 15 do
      Vga.setegacolor i;
      Vga.drawline (90 + i * 5) 10 (10 + i * 5) 90
    done ; 

    Vga.screenon ();

    let ys = ref 100 in
    let yw = (ymax - 100) / 4 in
      
    begin
      match Vga.getcolors() with
	  256 -> begin
	    for i = 0 to 59 do
	      let c = (i * 64) / 60 in
	    	Vga.setpalette (i + 16) c c c;
	    	Vga.setpalette (i + 16 + 60) c 0 0;
	    	Vga.setpalette (i + 16 + (2 * 60)) 0 c 0;
	    	Vga.setpalette (i + 16 + (3 * 60)) 0 0 c;
	    done ; 
	    lineset 0 15 ; lineset xmax 15;
	    lineset 1  0 ; lineset (xmax - 1) 0;
	    for x = 2 to xmax - 2 do
	      lineset x ((((x - 2) * 60) / (xmax - 3)) + 16)
	    done ; 
	    for y = !ys to !ys + yw - 1 do (* gray *)
	      Vga.drawscanline y line
	    done ; 
	    for x = 2 to xmax - 2 do
	      lineset x ((Char.code line.[x]) + 60)
	    done ; 
	    ys := !ys + yw;
	    for y = !ys to !ys + yw - 1 do (* red *)
	      Vga.drawscanline y line
	    done ;
	    for x = 2 to xmax - 2 do
	      lineset x ((Char.code line.[x]) + 60)
	    done ; 
	    ys := !ys + yw;
	    for y = !ys to !ys + yw - 1 do (* green *)
	      Vga.drawscanline y line
	    done ; 
	    for x = 2 to xmax - 2 do
	      lineset x ((Char.code line.[x]) + 60)
	    done ; 
	    ys := !ys + yw;
	    for y = !ys to !ys + yw - 1 do (* blue *)
	      Vga.drawscanline y line
	    done
	  end

      	| 32768 | 65536 | 16777216 ->
	    for x = 2 to xmax - 2 do
	      let c = ((x - 2) * 256) / (xmax - 3) in
	      let y = ref !ys in
	    	Vga.setrgbcolor c c c;
	    	Vga.drawline x !y x (!y + yw - 1);
	    	y := !y + yw;
	    	Vga.setrgbcolor c 0 0;
	    	Vga.drawline x !y x (!y + yw - 1);
	    	y := !y + yw;
	    	Vga.setrgbcolor 0 c 0;
	    	Vga.drawline x !y x (!y + yw - 1);
	    	y := !y + yw;
	    	Vga.setrgbcolor 0 0 c;
	    	Vga.drawline x !y x (!y + yw - 1)
	    done

      	| _ -> begin
	    if Vga.getcolors() = 16 then begin
	      for i = 0 to xmax - 2 do
		lineset i ((i + 2) mod 16)
	      done ; 
	      lineset 0 15 ; lineset xmax 15;
	      lineset 1  0 ; lineset (xmax - 1) 0;
	    end ; 
	    if Vga.getcolors() = 2 then begin
	      for i = 0 to xmax do
		lineset i 0x11
	      done ;
	      lineset 0 0x91
	    end ; 
	    for i = 100 to ymax - 2 do
	      Vga.drawscanline i line
	    done
          end
	      
    end ; 
      
    if getchar () = Char.code 'd' then
      Vga.dumpregs ();	

    Vga.setmode mode_TEXT
;;

let modes_list = [
  mode_G320x200x16  	;
  mode_G640x200x16  	;
  mode_G640x350x16  	;
  mode_G640x480x16  	;
  mode_G320x200x256 	;
  mode_G320x240x256 	;
  mode_G320x400x256 	;
  mode_G360x480x256 	;
  mode_G640x480x2   	;

  mode_G640x480x256 	;
  mode_G800x600x256 	;
  mode_G1024x768x256 	;

  mode_G1280x1024x256 ;	(* Additional modes. *)

  mode_G320x200x32K 	;
  mode_G320x200x64K 	;
  mode_G320x200x16M 	;
  mode_G640x480x32K 	;
  mode_G640x480x64K 	;
  mode_G640x480x16M 	;
  mode_G800x600x32K 	;
  mode_G800x600x64K 	;
  mode_G800x600x16M 	;
  mode_G1024x768x32K 	;
  mode_G1024x768x64K 	;
  mode_G1024x768x16M 	;
  mode_G1280x1024x32K ;
  mode_G1280x1024x64K ;
  mode_G1280x1024x16M ;

  mode_G800x600x16	;
  mode_G1024x768x16 	;
  mode_G1280x1024x16 	;

  mode_G720x348x2 	;		(* Hercules emulation mode *)

  mode_G320x200x16M32 ;		(* 32-bit per pixel modes. *)
  mode_G640x480x16M32 ;
  mode_G800x600x16M32 ;
  mode_G1024x768x16M32  ;
  mode_G1280x1024x16M32 ;

  mode_G1152x864x16 	;		(* additional resolutions *)
  mode_G1152x864x256 	;
  mode_G1152x864x32K 	;
  mode_G1152x864x64K 	;
  mode_G1152x864x16M 	;
  mode_G1152x864x16M32 ;

  mode_G1600x1200x16 	;
  mode_G1600x1200x256 ;
  mode_G1600x1200x32K ;
  mode_G1600x1200x64K ;
  mode_G1600x1200x16M ;
  mode_G1600x1200x16M32 ;

  mode_GLASTMODE
];;

let main () =

  Vga.init ();			(* Initialize. *)

  let cpt = ref 0
  and av_modes = ref [] in	

  print_string "Choose one of the following video modes: \n";

  List.iter 
    (fun modei ->
       if Vga.hasmode modei then begin
	 
	 let info = Vga.getmodeinfo modei in

	 let cols, expl =
	   (match info.colors with
	   	2 ->
		  "2", "1 bitplane, monochrome"
	      | 16 ->
		  "16", "4 bitplanes"
	      | 256 -> 
		  "", 
		  if modei = mode_G320x200x256 then 
		    "packed-pixel"
		  else if modei = mode_G320x240x256 or 
				  modei = mode_G320x400x256 or
					  modei = mode_G360x480x256 then
					    "Mode X"
		  else
		    "packed-pixel, banked"
	      | 32768 ->
		  "32K", "5-5-5 RGB, blue at LSB, banked"
	      | 65536 ->
		  "64K", "5-6-5 RGB, blue at LSB, banked"
	      | 16777216 ->
		  "16M", 
		  if info.bytesperpixel = 3 then
		    if List.mem  RGB_MISORDERED info.flags then
		      "8-8-8 BGR, red byte first, banked"
		    else
		      "8-8-8 RGB, blue byte first, banked"
		  else if List.mem RGB_MISORDERED info.flags then
		    "8-8-8 RGBX, 32-bit pixels, X byte first, banked"
		  else
		    "8-8-8 XRGB, 32-bit pixels, blue byte first, banked"
		    
  	      | _ -> "", ""
	   ) in
	   
	 let expl = expl ^ 
		    if List.mem IS_INTERLACED info.flags then
		      (if expl="" then "" else ", ") ^ "interlaced"
		    else "" in
	 let expl = expl ^
		    if List.mem IS_DYNAMICMODE info.flags then
		      (if expl="" then "" else ", ") ^ "dynamically loaded"
		    else "" in
	   
	   incr cpt ;
	   av_modes := (!cpt, modei) :: !av_modes ;
	   
	   Printf.printf "%2d %s: %dx%d, "
	     !cpt (string_of_mode modei) info.width info.height ;
	   if cols = "" then
	     Printf.printf "%d" info.colors
	   else
	     Printf.printf "%s" cols;
	   Printf.printf " colors ";
	   if expl <> "" then
	     Printf.printf "(%s)" expl;
	   Printf.printf "\n";
	   
       end) modes_list ;
    
    Printf.printf "Enter mode number (1-%d): " !cpt;
    
    let mode = 
      let n = int_of_string (read_line()) in
    	if n < 1 or n > !cpt then begin
	  Printf.printf "Error: Mode number out of range \n";
	  exit (-1)
    	end else
	  List.assoc n !av_modes 
    in
      if Vga.hasmode mode then
    	testmode mode
      else begin
    	Printf.printf "Error: Video mode not supported by driver\n" ;
    	exit (-1)
      end
;;

main ()

