/*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*/

/* $Id: gl_stub.c,v 1.11 1997/05/26 22:33:43 jc Exp $ */

#include <caml/mlvalues.h>
#include <caml/alloc.h>

#include <vga.h>
#include <vgamouse.h>
#include <vgagl.h>


value gl_currentcontext_stub(value u) {
  return (value) &currentcontext;
}

value gl_width_stub(value gc) {
  return Val_int(((GraphicsContext *) gc)->width);
}
value gl_height_stub(value gc) {
  return Val_int(((GraphicsContext *) gc)->height);
}
value gl_bytesperpixel_stub(value gc) {
  return Val_int(((GraphicsContext *) gc)->bytesperpixel);
}
value gl_colors_stub(value gc) {
  return Val_int(((GraphicsContext *) gc)->colors);
}
value gl_bitsperpixel_stub(value gc) {
  return Val_int(((GraphicsContext *) gc)->bitsperpixel);
}
value gl_bytewidth_stub(value gc) {
  return Val_int(((GraphicsContext *) gc)->bytewidth);
}



/* Configuration */

value null_graphics_context(value u) {
  return (value) NULL;
}

value gl_setcontextvga_stub(value mode) {
  gl_setcontextvga(Int_val(mode));
  return Val_unit;
}

value gl_setcontextvgavirtual_stub(value mode) {
  gl_setcontextvgavirtual(Int_val(mode));
  return Val_unit;
}

value gl_setcontextvirtual_stub(value w, value h, value bpp, value bitspp,
			       value vbuff) {
  gl_setcontextvirtual(Int_val(w), Int_val(h), Int_val(bpp), Int_val(bitspp),
		       String_val(vbuff));
  return Val_unit;
}

value gl_setcontextwidth_stub(value w) {
  gl_setcontextwidth(Int_val(w));
  return Val_unit;
}

value gl_setcontextheight_stub(value h) {
  gl_setcontextheight(Int_val(h));
  return Val_unit;
}

value gl_allocatecontext_stub(value unit) {
  GraphicsContext * gc;

  gc = gl_allocatecontext();
  return (value) gc;
}

value gl_setcontext_stub(value gc) {
  gl_setcontext((GraphicsContext *) gc);
  return Val_unit;
}

value gl_getcontext_stub(value gc) {
  gl_getcontext((GraphicsContext *) gc);
  return Val_unit;
}

value gl_freecontext_stub(value gc) {
  gl_freecontext((GraphicsContext *) gc);
  return Val_unit;
}


/* Line drawing */

value gl_setpixel_stub(value x, value y, value c) {
  gl_setpixel(Int_val(x), Int_val(y), Int_val(c));
  return Val_unit;
}

value gl_setpixelrgb_stub(value x, value y, value t) {
  gl_setpixelrgb(Int_val(x), Int_val(y), Int_val(Field(t,0)), 
		 Int_val(Field(t,1)), Int_val(Field(t,2)));
  return Val_unit;
}

value gl_getpixel_stub(value x, value y) {
  return Val_int(gl_getpixel(Int_val(x), Int_val(y)));
}

value gl_getpixelrgb_stub(value x, value y) {
  int red, green, blue;
  value result;
  
  gl_getpixelrgb(Int_val(x), Int_val(y), &red, &green, &blue);
  result = alloc(3,0);
  Field(result, 0) = Val_int(red);
  Field(result, 1) = Val_int(green);
  Field(result, 2) = Val_int(blue);
  return result;
}

value gl_rgbcolor_stub(value r, value g, value b) {
  return Val_int(gl_rgbcolor(Int_val(r), Int_val(g), Int_val(b)));
}

value gl_hline_stub(value x1, value y, value x2, value c) {
  gl_hline(Int_val(x1), Int_val(y), Int_val(x2), Int_val(c));
  return Val_unit;
}

value gl_line_stub(value x1, value y1, value x2, value y2, value c) {
  gl_line(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2), Int_val(c));
  return Val_unit;
}

value gl_circle_stub(value x, value y, value r, value c) {
  gl_circle(Int_val(x), Int_val(y), Int_val(r), Int_val(c));
  return Val_unit;
}


/* Box (bitmap) functions */

value gl_fillbox_stub(value x, value y, value w, value h, value c) {
  gl_fillbox(Int_val(x), Int_val(y), Int_val(w), Int_val(h), Int_val(c));
  return Val_unit;
}

value gl_getbox_stub(value x, value y, value w, value h, value dp) {
  gl_getbox(Int_val(x), Int_val(y), Int_val(w), Int_val(h), 
	    ((void *) String_val(dp)));
  return Val_unit;
}

value gl_putbox_stub(value x, value y, value w, value h, value dp) {
  gl_putbox(Int_val(x), Int_val(y), Int_val(w), Int_val(h), 
	    ((void *) String_val(dp)));
  return Val_unit;
}

value gl_putboxpart_stub_nat(value x, value y, value w, value h, value bw, 
			     value bh, value b, value xo, value yo) {
  gl_putboxpart(Int_val(x), Int_val(y), Int_val(w), Int_val(h),
		Int_val(bw), Int_val(bh), ((void *) String_val(b)),
		Int_val(xo), Int_val(yo));
  return Val_unit;
}

value gl_putboxpart_stub(value * argv, int argn) {
  return gl_putboxpart_stub_nat(argv[0], argv[1], argv[2], argv[3],
				argv[4], argv[5], argv[6], argv[7], argv[8]);
}

value gl_putboxmask_stub(value x, value y, value w, value h, value dp) {
  gl_putboxmask(Int_val(x), Int_val(y), Int_val(w), Int_val(h),
		((void *) String_val(dp)));
  return Val_unit;
}

value gl_copybox_stub_nat(value x1, value y1, value w, 
			  value h, value x2, value y2) {
  gl_copybox(Int_val(x1), Int_val(y1), Int_val(w), Int_val(h),
	     Int_val(x2), Int_val(y2));
  return Val_unit;
}

value gl_copybox_stub(value * argv, int argn) {
  return gl_copybox_stub_nat(argv[0], argv[1], argv[2], argv[3],
			     argv[4], argv[5]);
}

value gl_copyboxtocontext_stub_nat(value x1, value y1, value w, 
				   value h, value gc, value x2, value y2) {
  gl_copyboxtocontext(Int_val(x1), Int_val(y1), Int_val(w), Int_val(h),
		      (GraphicsContext *) gc, Int_val(x2), Int_val(y2));
  return Val_unit;
}

value gl_copyboxtocontext_stub(value * argv, int argn) {
  return gl_copyboxtocontext_stub_nat(argv[0], argv[1], argv[2], argv[3],
				      argv[4], argv[5], argv[6]);
}

value gl_copyboxfromcontext_stub_nat(value gc, value x1, value y1, value w, 
				     value h, value x2, value y2) {
  gl_copyboxfromcontext((GraphicsContext *) gc, Int_val(x1), Int_val(y1), 
			Int_val(w), Int_val(h), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value gl_copyboxfromcontext_stub(value * argv, int argn) {
  return gl_copyboxfromcontext_stub_nat(argv[0], argv[1], argv[2], argv[3],
				      argv[4], argv[5], argv[6]);
}

/* The following functions only work in 256-color modes: */

value gl_compileboxmask_stub(value w, value h, value sdp, value ddp) {
  gl_compileboxmask(Int_val(w), Int_val(h), ((void*) String_val(sdp)),
		    ((void *) String_val(ddp)));
  return Val_unit;
}

value gl_compiledboxmasksize_stub(value w, value h, value sdp) {
  return Val_int(gl_compiledboxmasksize(Int_val(w), Int_val(h), 
					((void *) String_val(sdp))));
}

value gl_putboxmaskcompiled_stub(value x, value y, 
				 value w, value h, value dp) {
  gl_putboxmaskcompiled(Int_val(x), Int_val(y), Int_val(w), Int_val(h),
			((void *) String_val(dp)));
  return Val_unit;
}


/* Miscellaneous */

value gl_clearscreen_stub(value c) {
  gl_clearscreen(Int_val(c));
  return Val_unit;     
}

value gl_scalebox_stub_nat(value w1, value h1, value sb,
			   value w2, value h2, value db) {
  gl_scalebox(Int_val(w1), Int_val(h1), ((void *) String_val(sb)),
	      Int_val(w2), Int_val(h2), ((void *) String_val(db)));
  return Val_unit;
}
value gl_scalebox_stub(value * argv, int argn) {
  return gl_scalebox_stub_nat(argv[0], argv[1], argv[2],
			      argv[3], argv[4], argv[5]);
}

value gl_setdisplaystart_stub(value x, value y) {
  gl_setdisplaystart(Int_val(x), Int_val(y));
  return Val_unit;
}

value gl_enableclipping_stub(value u) {
  gl_enableclipping();
  return Val_unit;
}

value gl_setclippingwindow_stub(value x1, value y1, value x2, value y2) {
  gl_setclippingwindow(Int_val(x1), Int_val(y1), Int_val(x2), Int_val(y2));
  return Val_unit;
}

value gl_disableclipping_stub(value u) {
  gl_disableclipping();
  return Val_unit;
}


/* Screen buffering */

value gl_copyscreen_stub(value gc) {
  gl_copyscreen((GraphicsContext *) gc);
  return Val_unit;
}

value gl_setscreenoffset_stub(value o) {
  gl_setscreenoffset(Int_val(o));
  return Val_unit;
}

value gl_enablepageflipping_stub(value gc) {
  return Val_int(gl_enablepageflipping((GraphicsContext *) gc));
}


/* Text */

value gl_expandfont_stub(value fw, value fh, value c, value sfdp, value dfdp) {
  gl_expandfont(Int_val(fw), Int_val(fh), Int_val(c),
		String_val(sfdp), String_val(dfdp));
  return Val_unit;
}

value gl_setfont_stub(value fw, value fh, value fdp) {
  gl_setfont(Int_val(fw), Int_val(fh), String_val(fdp));
  return Val_unit;
}

value gl_colorfont_stub(value fw, value fh, value c, value fdp) {
  gl_colorfont(Int_val(fw), Int_val(fh), Int_val(c), String_val(fdp));
  return Val_unit;
}

value gl_setwritemode_stub(value wm) {
  gl_setwritemode(Int_val(wm));
  return Val_unit;
}

value gl_write_stub(value x, value y, value s) {
  gl_write(Int_val(x), Int_val(y), String_val(s));
  return Val_unit;
}

value gl_writen_stub(value x, value y, value n, value s) {
  gl_writen(Int_val(x), Int_val(y), Int_val(n), String_val(s));
  return Val_unit;
}

value gl_setfontcolors_stub(value bg, value fg) {
  gl_setfontcolors(Int_val(bg), Int_val(fg));
  return Val_unit;
}

value gl_font8x8_stub(value u) {
  value r;
  
  r = copy_string(gl_font8x8);
  return r;
}


/* 256-color Palette */

value gl_setpalettecolor_stub(value c, value r, value g, value b) {
  gl_setpalettecolor(Int_val(c), Int_val(r), Int_val(g), Int_val(b));
  return Val_unit;
}

value gl_getpalettecolor_stub(value c) {
  int red, green, blue;
  value result;
  
  gl_getpalettecolor(Int_val(c), &red, &green, &blue);
  result = alloc(3,0);
  Field(result, 0) = Val_int(red);
  Field(result, 1) = Val_int(green);
  Field(result, 2) = Val_int(blue);
  return result;
}

value gl_setrgbpalette_stub(value unit) {
  gl_setrgbpalette();
  return Val_unit;
}
