(*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*)

(* $Id: keyboard.ml,v 1.3 1997/05/26 21:58:29 jc Exp $ *)


let scancode_ESCAPE	        = 1

let scancode_1		        = 2
let scancode_2			= 3
let scancode_3			= 4
let scancode_4			= 5
let scancode_5			= 6
let scancode_6			= 7
let scancode_7			= 8
let scancode_8			= 9
let scancode_9			= 10
let scancode_0			= 11

let scancode_MINUS		= 12
let scancode_EQUAL		= 13

let scancode_BACKSPACE		= 14
let scancode_TAB		= 15

let scancode_Q			= 16
let scancode_W			= 17
let scancode_E			= 18
let scancode_R			= 19
let scancode_T			= 20
let scancode_Y			= 21
let scancode_U			= 22
let scancode_I			= 23
let scancode_O			= 24
let scancode_P			= 25
let scancode_BRACKET_LEFT	= 26
let scancode_BRACKET_RIGHT	= 27

let scancode_ENTER		= 28

let scancode_LEFTCONTROL	= 29

let scancode_A			= 30
let scancode_S			= 31
let scancode_D			= 32
let scancode_F			= 33
let scancode_G			= 34
let scancode_H			= 35
let scancode_J			= 36
let scancode_K			= 37
let scancode_L			= 38
let scancode_SEMICOLON		= 39
let scancode_APOSTROPHE		= 40
let scancode_GRAVE	        = 41

let scancode_LEFTSHIFT		= 42
let scancode_BACKSLASH		= 43

let scancode_Z			= 44
let scancode_X			= 45
let scancode_C			= 46
let scancode_V			= 47
let scancode_B			= 48
let scancode_N			= 49
let scancode_M			= 50
let scancode_COMMA		= 51
let scancode_PERIOD		= 52
let scancode_SLASH		= 53

let scancode_RIGHTSHIFT		= 54
let scancode_KEYPADMULTIPLY	= 55

let scancode_LEFTALT		= 56
let scancode_SPACE		= 57
let scancode_CAPSLOCK		= 58

let scancode_F1			= 59
let scancode_F2			= 60
let scancode_F3			= 61
let scancode_F4			= 62
let scancode_F5			= 63
let scancode_F6		        = 64
let scancode_F7			= 65
let scancode_F8			= 66
let scancode_F9			= 67
let scancode_F10		= 68

let scancode_NUMLOCK		= 69
let scancode_SCROLLLOCK		= 70

let scancode_KEYPAD7		= 71
let scancode_CURSORUPLEFT	= 71
let scancode_KEYPAD8		= 72
let scancode_CURSORUP		= 72
let scancode_KEYPAD9		= 73
let scancode_CURSORUPRIGHT	= 73
let scancode_KEYPADMINUS	= 74
let scancode_KEYPAD4		= 75
let scancode_CURSORLEFT		= 75
let scancode_KEYPAD5		= 76
let scancode_KEYPAD6		= 77
let scancode_CURSORRIGHT	= 77
let scancode_KEYPADPLUS		= 78
let scancode_KEYPAD1		= 79
let scancode_CURSORDOWNLEFT	= 79
let scancode_KEYPAD2		= 80
let scancode_CURSORDOWN		= 80
let scancode_KEYPAD3		= 81
let scancode_CURSORDOWNRIGHT	= 81
let scancode_KEYPAD0		= 82
let scancode_KEYPADPERIOD	= 83

let scancode_LESS		= 86

let scancode_F11		= 87
let scancode_F12		= 88

let scancode_KEYPADENTER	= 96
let scancode_RIGHTCONTROL	= 97
let scancode_CONTROL		= 97
let scancode_KEYPADDIVIDE	= 98
let scancode_PRINTSCREEN	= 99
let scancode_RIGHTALT		= 100
let scancode_BREAK		= 101	(* Beware: is 119     *)
let scancode_BREAK_ALTERNATIVE	= 119	(* on some keyboards! *)

let scancode_HOME		= 102
let scancode_CURSORBLOCKUP	= 103	(* Cursor key block *)
let scancode_PAGEUP		= 104
let scancode_CURSORBLOCKLEFT	= 105	(* Cursor key block *)
let scancode_CURSORBLOCKRIGHT	= 106	(* Cursor key block *)
let scancode_END		= 107
let scancode_CURSORBLOCKDOWN	= 108	(* Cursor key block *)
let scancode_PAGEDOWN		= 109
let scancode_INSERT		= 110
let scancode_REMOVE		= 111


let key_NOTPRESSED  = false
let key_PRESSED     = true


external init : unit -> unit = "keyboard_init_stub"

type keyboard_handler = int -> bool -> unit
external raw_seteventhandler : unit -> unit = "keyboard_seteventhandler_stub"

let seteventhandler f =
  Callback.register "keyboard handler" f ;
  raw_seteventhandler ()
;;

external close : unit -> unit = "keyboard_close_stub"
external update : unit -> bool = "keyboard_update_stub"
external waitforupdate : unit -> unit = "keyboard_waitforupdate_stub"
external setdefaulteventhandler : unit -> unit 
  = "keyboard_setdefaulteventhandler_stub"
external getstate : unit -> string = "keyboard_getstate_stub"
external clearstate : unit -> unit = "keyboard_clearstate_stub"

type translate =
  | TRANSLATE_CURSORKEYS 	(* Map cursor block to keypad cursor. *)
  | TRANSLATE_DIAGONAL 	        (* Map keypad diagonal to keypad cursor. *)
  | TRANSLATE_KEYPADENTER 	(* Map keypad enter to main enter key. *)
  | DONT_CATCH_CTRLC            (* Disable Crtl-C check. *)

external raw_translatekeys : int -> unit = "keyboard_translatekeys_stub"

let translatekeys l =
  let rec decode = function 
      []                       -> 0
  | TRANSLATE_CURSORKEYS::rem  -> 1 lor (decode rem)
  | TRANSLATE_DIAGONAL::rem    -> 2 lor (decode rem)
  | TRANSLATE_KEYPADENTER::rem -> 4 lor (decode rem)
  | DONT_CATCH_CTRLC::rem      -> 8 lor (decode rem)
  in
    raw_translatekeys (decode l)
;;

external keypressed : int -> bool = "keyboard_keypressed_stub"
