(*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*)

(* $Id: mouse.ml,v 1.3 1997/05/26 21:58:29 jc Exp $ *)

type mouse_type = int

let mouse_MICROSOFT = 0
let mouse_MOUSESYSTEMS = 1
let mouse_MMSERIES = 2
let mouse_LOGITECH = 3
let mouse_BUSMOUSE = 4
let mouse_PS2 = 5
let mouse_LOGIMAN = 6

let mouse_CHG_DTR	= 0x80000000	(* CLEAR (!) the DTR line *)
let mouse_DTR_HIGH	= 0x40000000	(* when used with MOUSE_CHG_DTR 
					   set DTR not clear it *)
let mouse_CHG_RTS	= 0x20000000	(* CLEAR (!) the RTS line *)
let mouse_RTS_HIGH	= 0x10000000	(* when used with MOUSE_CHG_RTS 
					   set RTS not clear it *)

let mouse_type_mask = 0xffff

let string_of_mouse_type mt = match mt land mouse_type_mask with
    0 -> "MICROSOFT"
  | 1 -> "MOUSESYSTEMS"
  | 2 -> "MMSERIES"
  | 3 -> "LOGITECH"
  | 4 -> "BUSMOUSE"
  | 5 -> "PS2"
  | 6 -> "LOGIMAN"
  | _ -> invalid_arg "string_of_mouse_type"
;;

let defaultsamplerate = 150
  
external init : string -> mouse_type -> int -> unit = "mouse_init_stub"

type mouse_handler = int -> int -> int -> unit

external raw_seteventhandler : unit -> unit = "mouse_seteventhandler_stub"

let seteventhandler f =
  Callback.register "mouse handler" f ;
  raw_seteventhandler ()
;;

external close : unit -> unit = "mouse_close_stub"
external update : unit -> bool = "mouse_update_stub"
external waitforupdate : unit -> unit = "mouse_waitforupdate_wtub"


external setdefaulteventhandler : unit -> unit = 
		                         "mouse_setdefaulteventhandler_stub"
external setposition : int -> int -> unit = "mouse_setposition_stub"
external setxrange : int -> int -> unit = "mouse_setxrange_stub"
external setyrange : int -> int -> unit = "mouse_setyrange_stub"
external setscale : int -> unit = "mouse_setscale_stub"

type wrap = int

let wrap_NOWRAP = 0
let wrap_WRAPX  = 1
let wrap_WRAPY  = 2
let wrap_WRAP   = 3

external setwrap : int -> unit = "mouse_setwrap_stub"
external getx : unit -> int = "mouse_getx_stub"
external gety : unit -> int = "mouse_gety_stub"

let button_LEFTBUTTON = 4
let button_MIDDLEBUTTON = 2
let button_RIGHTBUTTON = 1

external getbutton : unit -> int = "mouse_getbutton_stub"
