(*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*)

(* $Id: mouse.mli,v 1.4 1997/05/26 21:58:30 jc Exp $ *)

type mouse_type = int

val mouse_MICROSOFT : mouse_type
val mouse_MOUSESYSTEMS : mouse_type
val mouse_MMSERIES : mouse_type
val mouse_LOGITECH : mouse_type
val mouse_BUSMOUSE : mouse_type
val mouse_PS2 : mouse_type
val mouse_LOGIMAN : mouse_type

(* Do a logical OR between one of the above modes and the following values,
   and give the result as mouse_type in the call to the "init" function. *)

val mouse_CHG_DTR	: int	(* CLEAR (!) the DTR line *)
val mouse_DTR_HIGH	: int	(* when used with MOUSE_CHG_DTR
				   set DTR not clear it *)
val mouse_CHG_RTS	: int   (* CLEAR (!) the RTS line *)
val mouse_RTS_HIGH	: int	(* when used with MOUSE_CHG_RTS
				   set RTS not clear it *)

val mouse_type_mask : int

val string_of_mouse_type : mouse_type -> string


val defaultsamplerate : int


(* Initialize mouse device.
   Raises (Failure "Mouse initialization failed") if unsuccessful.
   (Get the svgalib configured type with Vga.getmousetype()). *)
  external init : string -> mouse_type -> int -> unit = "mouse_init_stub"

(* Set event handler invoked by Mouse.update(). *)
  type mouse_handler = int -> int -> int -> unit
  val seteventhandler : mouse_handler -> unit

(* Close mouse device. *)
  external close : unit -> unit = "mouse_close_stub"

(* Read mouse and handle events. Returns [false] if no event. *)
  external update : unit -> bool = "mouse_update_stub"

(* Similar to mouse_update, but wait for an event to happen. *)
  external waitforupdate : unit -> unit = "mouse_waitforupdate_wtub"



(* Revert to default handler. *)
  external setdefaulteventhandler : unit -> unit = 
		                         "mouse_setdefaulteventhandler_stub"

(* Set position of mouse. *)
   external setposition : int -> int -> unit = "mouse_setposition_stub"

(* Set horizontal range of mouse to [x1, x2] incl. *)
   external setxrange : int -> int -> unit = "mouse_setxrange_stub"

(* Set vertical range of mouse to [y1, y2] incl. *)
   external setyrange : int -> int -> unit = "mouse_setyrange_stub"

(* Set scale factor by which raw mouse coordinates are divided. *)
   external setscale : int -> unit = "mouse_setscale_stub"

type wrap

val wrap_NOWRAP : wrap
val wrap_WRAPX  : wrap
val wrap_WRAPY  : wrap
val wrap_WRAP   : wrap

(* Enable/disable wrapping of mouse in horizontal and/or vertical range. *)
   external setwrap : wrap -> unit = "mouse_setwrap_stub"

(* Get current mouse x-coordinate. *)
   external getx : unit -> int = "mouse_getx_stub"

(* Get current mouse y-coordinate. *)
   external gety : unit -> int = "mouse_gety_stub"

val button_LEFTBUTTON : int
val button_MIDDLEBUTTON : int
val button_RIGHTBUTTON : int

(*  Get current mouse button status. *)
   external getbutton : unit -> int = "mouse_getbutton_stub"




