/*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*/

/* $Id: vga_stub.c,v 1.14 1997/05/28 19:02:06 jc Exp $ */

#include <caml/mlvalues.h>
#include <caml/alloc.h>

#include <vga.h>
#include <vgamouse.h>
#include <vgagl.h>


/* initialization ******************************************************/

value vga_init_stub(value unit)
{
  static char *vga_init_message = "Not a graphics-capable virtual console";

  if (vga_init())
    failwith(vga_init_message);
  return Val_unit;
}

value vga_clear_stub(value unit)
{
  vga_clear();
  return Val_unit;
}


/* modes *******************************************************/

value vga_getmodenumber_stub(value name) {
  int m;
  static char *vga_getmode_number_message = "getmodenumber";

  if ( strcmp (String_val(name),"TEXT") == 0) {
    return TEXT;
  } else {
    m = vga_getmodenumber(String_val(name));
    if (m == -1)
      invalid_argument(vga_getmode_number_message);
    return Val_int(m);
  }
}

value vga_getmodename_stub(value mode) {
  char *name;
  int m;

  m = Int_val(mode);
  if (m == 0) {
    return copy_string("TEXT");
  } else {
    name = vga_getmodename(Int_val(mode));
    return copy_string(name);
  }
}

value vga_hasmode_stub(value mode) {
  return Val_bool(vga_hasmode(Int_val(mode)));
}

value vga_setmode_stub(value mode) {
  vga_setmode(Int_val(mode));
  return Val_unit;
}

value vga_getdefaultmode_stub(value unit) {
  static char *vga_getdefaultmode_message = "Invalid mode in GSVGAMODE";
  int m;

  m = vga_getdefaultmode();
  if (m == -1)
    failwith(vga_getdefaultmode_message);
  return Val_int(m);
}

value vga_getcurrentmode_stub(value unit) {
  return Val_int(vga_getcurrentmode());
}

value vga_lastmodenumber_stub(value unit) {
  return Val_int(vga_lastmodenumber());
}

value vga_getcurrentchipset_stub(value unit) {
  return Val_int(vga_getcurrentchipset());
}

value vga_getxdim_stub(value unit) {
  return Val_int(vga_getxdim());
}
value vga_getydim_stub(value unit) {
  return Val_int(vga_getydim());
}
value vga_getcolors_stub(value unit) {
  return Val_int(vga_getcolors());
}

value call_convert_blit_flags(int f) {
  static value * closure_convert_blit_flags = NULL;

  if (closure_convert_blit_flags == NULL) {
    closure_convert_blit_flags = 
      (value *) caml_named_value("convert_blit_flags");
  }
  return callback(*closure_convert_blit_flags, Val_int(f));
}

value call_convert_other_flags(int f) {
  static value * closure_convert_other_flags = NULL;

  if (closure_convert_other_flags == NULL) {
    closure_convert_other_flags =
      (value *) caml_named_value("convert_other_flags");
  }
  return callback(*closure_convert_other_flags, Val_int(f));
}

value vga_getmodeinfo_stub(value mode) {
  vga_modeinfo *modeinfo;
  value result;

  modeinfo = vga_getmodeinfo(Int_val(mode));
  result = alloc(13,0);
  Field(result, 0) = Val_int(modeinfo->width);
  Field(result, 1) = Val_int(modeinfo->height);
  Field(result, 2) = Val_int(modeinfo->bytesperpixel);
  Field(result, 3) = Val_int(modeinfo->colors);
  Field(result, 4) = Val_int(modeinfo->linewidth);
  Field(result, 5) = Val_int(modeinfo->maxlogicalwidth);
  Field(result, 6) = Val_int(modeinfo->startaddressrange);
  Field(result, 7) = Val_int(modeinfo->maxpixels);
  Field(result, 8) = call_convert_blit_flags(modeinfo->haveblit);
  Field(result, 9) = call_convert_other_flags(modeinfo->flags);
  Field(result,10) = Val_int(modeinfo->chiptype);
  Field(result,11) = Val_int(modeinfo->memory);
  Field(result,12) = Val_int(modeinfo->linewidth_unit);
  return result;
}

/* functions to set some of the chipset parameters **********************/

value vga_setpage_stub(value p) {
  vga_setpage(Int_val(p));
  return Val_unit;
}
value vga_setreadpage_stub(value p) {
  vga_setreadpage(Int_val(p));
  return Val_unit;
}
value vga_setwritepage_stub(value p) {
  vga_setwritepage(Int_val(p));
  return Val_unit;
}
value vga_setlogicalwidth_stub(value p) {
  vga_setlogicalwidth(Int_val(p));
  return Val_unit;
}
value vga_setdisplaystart_stub(value p) {
  vga_setdisplaystart(Int_val(p));
  return Val_unit;
}

value vga_claimvideomemory_stub(value m) {
  static char *vga_claimvideomemory_message = "Not enough memory on the card";
  int ok;
  
  ok = vga_claimvideomemory(Int_val(m));
  if (ok == -1)
    failwith(vga_claimvideomemory_message);
  return Val_unit;
}

value vga_setmodeX_stub(value unit) {
  if (vga_setmodeX())
    return Val_unit;
  else
    failwith("mode X not available");
}
value vga_disabledriverreport_stub(value unit) {
  vga_disabledriverreport();
  return Val_unit;
}
value vga_getmousetype_stub(value unit) {
  return(Val_int(vga_getmousetype()));
}
value vga_getmonitortype_stub(value unit) {
  return(Val_int(vga_getmonitortype()));
}


/* refresh on/off ******************************************************/

value vga_screenoff_stub(value unit) {
  vga_screenoff();
  return Val_unit;
}
value vga_screenon_stub(value unit) {
  vga_screenon();
  return Val_unit;
}
value vga_waitretrace_stub(value unit) {
  vga_waitretrace();
  return Val_unit;
}

/* flipping ************************************************************/

value vga_setflipchar_stub(value c) {
  vga_setflipchar(Int_val(c));
  return Val_unit;
}

value vga_flip_stub(value unit) {
  vga_flip();
  return Val_unit;
}

value vga_getch_stub(value unit) {
  return Val_int(vga_getch());
}


/* drawing functions ****************************************************/

value vga_setcolor_stub(c)
     value c;
{
  vga_setcolor(Int_val(c));
  return Val_unit;
}

value vga_drawpixel_stub(x, y)
     value x,y;
{
  vga_drawpixel(Int_val(x), Int_val(y));
  return Val_unit;
}

value vga_drawline_stub(value x1, value y1, value x2, value y2) {
  vga_drawline(Int_val(x1),Int_val(y1),Int_val(x2),Int_val(y2));
  return Val_unit;
}

value vga_getpixel_stub(value x, value y) {
  return Val_int(vga_getpixel(Int_val(x), Int_val(y)));
}

value vga_drawscanline_stub(value line, value colors) {
  vga_drawscanline(Int_val(line), ((unsigned char *) String_val(colors)));
  return Val_unit;
}

value vga_drawscansegment_stub(value colors, value x, value y, value length) {
  vga_drawscansegment(((unsigned char *) String_val(colors)), 
		      Int_val(x), Int_val(y), Int_val(length));
  return Val_unit;
}

value vga_getscansegment_stub(value colors, value x, value y, value length) {
  static char *vga_getscansegment_message = "Not supported";

  if (vga_getscansegment(((unsigned char *) String_val(colors)), 
			 Int_val(x), Int_val(y), Int_val(length)))
    failwith (vga_getscansegment_message);
  return Val_unit;
}


/* palette **************************************************************/

value vga_setpalette_stub(value index, value red, value green, value blue) {
  vga_setpalette(Int_val(index), Int_val(red), Int_val(green), Int_val(blue));
  return Val_unit;
}

value vga_getpalette_stub(value index) {
  int red, green, blue;
  value result;
  
  vga_getpalette(Int_val(index), &red, &green, &blue);
  result = alloc(3,0);
  Field(result, 0) = Val_int(red);
  Field(result, 1) = Val_int(green);
  Field(result, 2) = Val_int(blue);
  return result;
}

/* For vga_setpalvec and vga_getpalvec, we expand the C functions,
   in order not to have to allocate a vector again */

value vga_setpalvec_stub(value s, value num, value palvec) {
  int i,j,start;
  
  start = Int_val(s);
  j = 0;
  for (i = start; i < start + (Int_val(num)); ++i) {
	vga_setpalette(i, Int_val(Field(palvec,j++)),
		          Int_val(Field(palvec,j++)), 
		          Int_val(Field(palvec,j++)));
    }
  return Val_unit;
}

value vga_getpalvec_stub(value s, value num, value palvec) {
  int i,j,start,red,green,blue;
  
  start = Int_val(s);
  j = 0;
  for (i = start; i < start + (Int_val(num)); ++i) {
    vga_getpalette(i, &red, &green, &blue);
    Field(palvec,j++) = Val_int(red);
    Field(palvec,j++) = Val_int(green);
    Field(palvec,j++) = Val_int(blue);
  }
  return Val_unit;
}

value vga_white_stub(value unit) {
  return Val_int(vga_white());
}

value vga_setegacolor_stub(value color) {
  vga_setegacolor(Int_val(color));
  return Val_unit;
}

value vga_setrgbcolor_stub(value red, value green, value blue) {
  vga_setrgbcolor(Int_val(red), Int_val(green), Int_val(blue));
  return Val_unit;
}


/* keyboard **************************************************************/

value vga_getkey_stub(unit)
     value unit;
{
  return Val_int(vga_getkey());
}


/* dump regs **************************************************************/

value vga_dumpregs_stub(value unit) {
  vga_dumpregs();
  return Val_unit;
}

/*** Mouse module *********************************************************/

value mouse_init_stub(value dev, value type, value samplerate) {
  static char *mouse_init_message = "Mouse initialization failed";
  int ok;

  ok = mouse_init(String_val(dev), Int_val(type), Int_val(samplerate));
  if (ok == -1)
    failwith(mouse_init_message);
  return Val_unit;
}

/* Hack to pass the mouse handler (a Caml function) to mouse_seteventhandler,
   which takes a C function: we have a pointer closure_mouse_handler on the
   closure of the current Caml mouse handler, then a C function mouse_handler
   that calls that closure and which is the real mouse handler. */

static value * closure_mouse_handler = NULL;

void mouse_handler(int button, int dx, int dy) {
  callback3(*closure_mouse_handler, Val_int(button), Val_int(dx), Val_int(dy));
}

value mouse_seteventhandler_stub(value unit) {
  closure_mouse_handler = (value *) caml_named_value("mouse handler");
  mouse_seteventhandler(mouse_handler);
  return Val_unit;
}

value mouse_close_stub(value unit) {
  mouse_close();
  return Val_unit;
}

value mouse_update_stub(value unit) {
  if (mouse_update())
    return Val_int(1);
  else
    return Val_int(0);
}

value mouse_waitforupdate_stub(value unit) {
  mouse_waitforupdate();
  return Val_unit;
}


value mouse_setdefaulteventhandler_stub(value unit) {
  mouse_setdefaulteventhandler();
  return Val_unit;
}

value mouse_setposition_stub(value x, value y) {
  mouse_setposition(Int_val(x), Int_val(y));
  return Val_unit;
}

value mouse_setxrange_stub(value x1, value x2) {
  mouse_setxrange(Int_val(x1), Int_val(x2));
  return Val_unit;
}

value mouse_setyrange_stub(value y1, value y2) {
  mouse_setyrange(Int_val(y1), Int_val(y2));
  return Val_unit;
}

value mouse_setscale_stub(value s) {
  mouse_setscale(Int_val(s));
  return Val_unit;
}

value mouse_setwrap_stub(value w) {
  mouse_setwrap(Int_val(w));
  return Val_unit;
}

value mouse_getx_stub(value unit) {
  return Val_int(mouse_getx());
}

value mouse_gety_stub(value unit) {
  return Val_int(mouse_gety());
}

value mouse_getbutton_stub(value unit) {
  return Val_int(mouse_getbutton());
}

/*** Keyboard module *********************************************************/

value keyboard_init_stub(value u) {
  static char *keyboard_init_message = "Keyboard initialization failed";
  int ok;

  ok = keyboard_init();
  if (ok == -1)
    failwith(keyboard_init_message);
  return Val_unit;
}

/* Hack to pass the keyboard handler (a Caml function) to
   keyboard_seteventhandler, which takes a C function: we have a
   pointer closure_keyboard_handler on the closure of the current Caml
   keyboard handler, then a C function keyboard_handler that calls
   that closure and which is the real keyboard handler. */

static value * closure_keyboard_handler = NULL;

void keyboard_handler(int button, int dx, int dy) {
  callback3(*closure_keyboard_handler, 
	    Val_int(button), Val_int(dx), Val_int(dy));
}

value keyboard_seteventhandler_stub(value unit) {
  closure_keyboard_handler = (value *) caml_named_value("keyboard handler");
  keyboard_seteventhandler(keyboard_handler);
  return Val_unit;
}

value keyboard_close_stub(value unit) {
  keyboard_close();
  return Val_unit;
}

value keyboard_update_stub(value unit) {
  if (keyboard_update())
    return Val_int(1);
  else
    return Val_int(0);
}

value keyboard_waitforupdate_stub(value unit) {
  keyboard_waitforupdate();
  return Val_unit;
}

value keyboard_setdefaulteventhandler_stub(value unit) {
  keyboard_setdefaulteventhandler();
  return Val_unit;
}

value keyboard_getstate_stub(value u) {
  value result;
  int i;
  char * state;
  
  state = (char*) keyboard_getstate();
  result = alloc_string(128);
  for (i = 0; i < 128 ; i++) {
    Byte(result, i) = state[i];
  }
  return result;
}

value keyboard_clearstate_stub(value u) {
  keyboard_clearstate();
  return Val_unit;
}

value keyboard_translatekeys_stub(value m) {
  keyboard_translatekeys(Int_val(m));
  return Val_unit;
}

value keyboard_keypressed_stub(value scancode) {
  if (keyboard_keypressed(Int_val(scancode)))
    return Val_true;
  else
    return Val_false;
}
